# option_dialog.rb

# $Id: option_dialog.rb,v 1.1.1.1 2002/04/30 04:15:45 kenji Exp $

# 2002/4/30  Takuro Ashie <ashie@homa.ne.jp>
# Copyright (C) 2002 Takuro Ashie <ashie@homa.ne.jp>
# This is free software with ABSOLUTELY NO WARRANTY.

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.


###############################################################################
#
#   OptionDialog Widget
#
###############################################################################
class OptionDialog < Gtk::Dialog
  include Gtk

  #
  # initialize ()
  #
  #   aMessage  : message to show
  #   aXpm      : Gtk::Pixmap object to show
  #   aPosition : window position
  #   Return    : a value which will be attached to button.
  #
  def initialize(aTitle, aMessage, aXpm, aPosition)
    super()
    realize
    set_title(aTitle)
    set_modal(true)
    set_position(aPosition)

    @label     = nil     # Gtk::Label
    @buttonbox = nil     # Gtk::HButtonBox
    @result    = false   # Bool

    createChildren (aMessage, aXpm)
  end

  attr_reader :result


 private

  #
  # createChildren ()
  #
  #   aMessage : message to show
  #   aXpm     : a Gtk::Pixmap to show
  #
  def createChildren (aMessage, aXpm)
    @label = Label::new (aMessage)
    @label.set_justify (JUSTIFY_LEFT)
    hbox = HBox::new
    if (aXpm) then
      hbox.pack_start (createPixmapWidget (aXpm),
		       false, false, 10)
    end
    hbox.pack_start (@label, true, true, 10)
    vbox.pack_start (hbox, true, true, 10)
    vbox.border_width (10)

    @buttonbox = HButtonBox::new
    @buttonbox.set_layout (BUTTONBOX_END)
    action_area.pack_start (@buttonbox, true, true, 0)
  end

  #
  # createPixmapWidget (pixmap_data)
  #
  #   aPixmapData : 
  #
  def createPixmapWidget (aPixmapData)
    pixmap, mask = Gdk::Pixmap::create_from_xpm_d (window,
						   style.bg(Gtk::STATE_NORMAL),
						   aPixmapData)
    return Pixmap::new(pixmap, mask)
  end


 protected

  #
  # addButton ()
  #   append a button to the button area.
  #
  #   aName  : Label for this button.
  #   aFlag  : flag to attach this button (will be treated as return value of
  #            the dialog)
  #   Return : a New Gtk:Button widget.
  #
  def addButton (aName, aFlag)
    button = Button::new(aName)
    button.set_flags(Widget::CAN_DEFAULT)
    button.signal_connect(Gtk::Button::SIGNAL_CLICKED) {
      @result = aFlag
      Gtk.main_quit
    }
    @buttonbox.add(button)
    return button
  end


 public

  #
  # popup ()
  #   show window, and pause program.
  #
  def popup ()
    show_all

    Gtk.main
    destroy

    return @result
  end
end



###############################################################################
#
#   ErrorDialog Widget
#
###############################################################################
class ErrorDialog < OptionDialog

XPM_ERROR = [
"    48    48      134            2",
".. c None",
".# c #c47a2c",
".a c #841e0c",
".b c #c42614",
".c c #a42214",
".d c #440e0c",
".e c #94322c",
".f c #c49e34",
".g c #a43224",
".h c #d4aa2c",
".i c #842a1c",
".j c #e42a14",
".k c #b42214",
".l c #d4aa44",
".m c #64221c",
".n c #941e14",
".o c #942a1c",
".p c #d4a234",
".q c #a43a2c",
".r c #340e04",
".s c #d42a14",
".t c #a42a1c",
".u c #4c1a14",
".v c #cca234",
".w c #e4ae24",
".x c #f43214",
".y c #b42a1c",
".z c #742624",
".A c #942614",
".B c #140604",
".C c #8c3a34",
".D c #8c2a1c",
".E c #741a0c",
".F c #dcb23c",
".G c #3c0e04",
".H c #ac2214",
".I c #ac3224",
".J c #dcb234",
".K c #9c2a1c",
".L c #ac3a2c",
".M c #84261c",
".N c #c42e1c",
".O c #441614",
".P c #dcaa2c",
".Q c #ec321c",
".R c #bc2214",
".S c #dc2a14",
".T c #ac2a1c",
".U c #5c1a14",
".V c #cc8624",
".W c #cc2a14",
".X c #9c3224",
".Y c #ec2e14",
".Z c #9c1e14",
".0 c #dca634",
".1 c #cca644",
".2 c #9c2614",
".3 c #1c0604",
".4 c #8c2e2c",
".5 c #7c1e0c",
".6 c #dcae2c",
".7 c #0c0204",
".8 c #c42a14",
".9 c #a42614",
"#. c #a4362c",
"## c #842e24",
"#a c #6c1e14",
"#b c #240a04",
"#c c #b42614",
"#d c #6c221c",
"#e c #942e24",
"#f c #d4a634",
"#g c #a43e34",
"#h c #34120c",
"#i c #a42e24",
"#j c #cca63c",
"#k c #fc3214",
"#l c #bc2a1c",
"#m c #9c3a34",
"#n c #8c2e24",
"#o c #3c1214",
"#p c #ac2614",
"#q c #ac3624",
"#r c #9c2e24",
"#s c #ac3e2c",
"#t c #dcae3c",
"#u c #dc2e14",
"#v c #1c0a04",
"#w c #c4823c",
"#x c #c49e3c",
"#y c #a4322c",
"#z c #842a24",
"#A c #6c160c",
"#B c #e42e1c",
"#C c #d4aa4c",
"#D c #942214",
"#E c #942a24",
"#F c #a43a34",
"#G c #340e0c",
"#H c #a42a24",
"#I c #541a14",
"#J c #cca23c",
"#K c #e4b234",
"#L c #94261c",
"#M c #8c2a24",
"#N c #741e14",
"#O c #3c120c",
"#P c #9c2a24",
"#Q c #8c261c",
"#R c #4c1614",
"#S c #dcaa34",
"#T c #2c0e0c",
"#U c #bc2614",
"#V c #d48e1c",
"#W c #7c2a24",
"#X c #cc9e34",
"#Y c #9c2214",
"#Z c #0c0604",
"#0 c #6c261c",
"#1 c #9c322c",
"#2 c #ec2e1c",
"#3 c #9c261c",
"#4 c #dcae34",
"#5 c #c42a1c",
"#6 c #a4261c",
"#7 c #44120c",
"#8 c #240a0c",
"#9 c #d4a63c",
"a. c #7c221c",
"a# c #ac261c",
"aa c #ac362c",
"ab c #ac3e34",
"ac c #dcae44",
"ad c #5c1e1c",
"................................................................................................",
"................................................................................................",
"................................................................................................",
"................................................................................................",
"................................................................................................",
"................................................................................................",
"................................................................................................",
"................................................................................................",
".....................................d.a#U.S.Y#2#B.W#p.5.d......................................",
"...............................3.a#B.Y.s.b#U#c#c#p#c#c#l.8#l#A.B................................",
".............................5#k.S.b.k.H.H.c.c.n.n.c.9.9a##p.y#l.U..............................",
"........................#b#U.Y.b.k.H.H.c.H.c#Y.c#p.9#p#6.9.t.9.T.T.a.3..........................",
".......................G.Y.s.R.H.c.H.c.H.c#D.c.c.9.c#6.9#6.9#6.t.t.T#3#b........................",
".....................r.x.W.k.H.c.H.c.H.c.c.Z.c.9#p.9a#.I.I#6.t.t#3.t.t#L.3......................",
".....................s.s.k.c.H.c.H.c.H.9#p.9#Y.c.9#6.9.t#qaa.t.t#3.t.K.K#N......................",
"..................#Y.j.k.H.H.c.H.c#p.T.I.c.9.9#Y.2.9#6#6#6aa.Iaa.t#H.K#H.K#R....................",
".................G#k#U.H.H.c.H.c.T.I.9.I.I.9#p#6#3#D#3.t.t.t.t#.aa.t#i.K#P#Q.3..................",
"................#p.s.H.H.c.H.c#p#D.K.I#p.I.I#6.9#6.t#D.A#6.t.t.taa.q.t#i.t.o#R..................",
"...............3#k#U.c.H.c.n#p.c#Y.9.9.I.I.I.I.t.T.t#6#L#D#3#H#i.t#F.L#r#r#Pa.#Z................",
"...............E#u.H.H.c.c#Y.9#p.9#p.9.9.I.Iab#q#6.t.t.t#L#L.K.t#i#H.q#s#i#i.i#b................",
"..............#p.b#p#V.w.w.w.w.w.6.w.6.6.P.6.J#K.F#S#S.h#S.p.f.v#f#f#9.##r#r#E#O................",
"..............#u#U.c.w.w.w.w.w.w.6.6.w.P.P.6#Sac#t#t.h.0#f#f#f.p#9#f#9#j#i#i.o.O................",
"..............#B#c#p.w.w.w.6.6.6.6.w.P.6.6.P.P#f.v#tac#f.h.0#f#9#f#9#j.p#y#r#e.U................",
"...............Q#c.9.w.w.w.w.J.w.6.P.6.6.P.6.h.P.p#X#fac.l#9#9#f#x.f#x#x#i.X#e#I................",
"..............#u#c.9.6.w.h.6.6.w.J.6.P.6.P.h#S#f.h#f#X.p.l.l.l#f#9.p#J#9#1#r#e.u................",
"..............#5#c.9.w.6.6.w.6.6#t.J.P#4#f#4#X#f.0#f#f#X#j#f.l.l.1#9.1#J.g#1#M#O................",
"..............#Y.y.9.V.6.w.6.P.6.P.F#t#S#S#f#X#X#f#f#f.p.v#J#9#C.l#j.1#w#1#r#W#8................",
"..............#A#l.9.9.9.t.9.t.2#3.t#q#.#i.t#i.K#P#i#r#E#i#r#r#r#y#F#1.g#1.e#d.7................",
"...............3.N.T.9.t.9.t.t.t#3#L.K#..t#i#i#r.D.D#i#e#i#r.X.g#1.g#g#1#1#M.u..................",
"................#D.T.t.9.T.t.t.t#i#3.K.K#.#r.t#i#i.M.D#r#e#y#i#r.X#y#1#g.X.z#v..................",
".................r.y.T.t#6.t.t.t.t#i.K#H#.#1#r#r#i#r#r#i#e#i#1#r#1#y#1#g.C.u....................",
"...................E.T.t.t.t#H.t#i.t.K#e#i.q#i#r#r#i.X#y#e#e#y#1.g#1#1.4.m#Z....................",
"....................#L.t#3#H.t#i.K#i#i.o#r#i.q#i.g#1.g#r#y#e#1#.#1#1.e.z#T......................",
"....................#b#3#P.t.K#i#i#r#i#i#i.g#F#m#E#e#y#1#y.e#1#1#1.ea.#h........................",
"......................#b#Q.K#P.t#r#i#i#r#i.X#r.q.C#P.X#y.X#1.X#1#n#0#h..........................",
".........................B#a.o#P.K#i#r#i.g#r.g#e#m#g#y#1#1#1.e##ad#v............................",
"............................#O.M#E.o#r#r#r#y#r#1.X#g#1.X.e#z#d#o#Z..............................",
"...............................7#G#a.M#M#M#e#e#e#e#M#z.zad#h#Z..................................",
".....................................B#T#7.u.U#I.u#o#T.B........................................",
"................................................................................................",
"................................................................................................",
"................................................................................................",
"................................................................................................",
"................................................................................................",
"................................................................................................",
"................................................................................................",
"................................................................................................",
"................................................................................................"
]

  #
  # initialize ()
  #
  #   aMessage  : message to show.
  #   aTitle    :
  #   aPosition : window position.
  #
  def initialize (aMessage, aTitle = "顼", aPosition = WIN_POS_MOUSE)
    super (aTitle, aMessage, XPM_ERROR, aPosition)

    button = addButton ("ǧ", true)
    button.grab_default 
  end
end



###############################################################################
#
#   QuestionDialog Widget
#
###############################################################################
class QuestionDialog < OptionDialog

XPM_QUESTION = [
"48 48 74 1",
" 	c None",
".	c #000000",
"+	c #A27F3B",
"@	c #B89243",
"#	c #D8AB4E",
"L	c #9D7D3A",
"J	c #A6823C",
"K	c #7C622D",
"*	c #F7D899",
"=	c #F7E2B8",
"-	c #D8BA7B",
";	c #BE9B53",
">	c #F7C35A",
",	c #F7DFAF",
"'	c #F7E5C0",
")	c #F7D48C",
"!	c #F7CF7E",
"~	c #D8B15F",
"{	c #6E5526",
"]	c #F7DCA5",
"^	c #A8843D",
"/	c #937435",
"(	c #AA863E",
"_	c #D8B66E",
":	c #BEA363",
"<	c #947536",
"[	c #927335",
"}	c #F7C96D",
"|	c #80632D",
"1	c #C39F57",
"2	c #A38E55",
"3	c #A48B5A",
"4	c #D2A64D",
"5	c #82662F",
"6	c #654F24",
"7	c #DCB463",
"8	c #BE9544",
"9	c #C3A466",
"0	c #8C6D31",
"a	c #D2B06C",
"b	c #D8BE86",
"c	c #715929",
"d	c #B99344",
"e	c #B38D40",
"f	c #D8B56E",
"g	c #D8AC58",
"h	c #997A3B",
"i	c #000001",
"j	c #6B5527",
"k	c #C39F55",
"l	c #DCC494",
"m	c #DAB872",
"n	c #C9AC73",
"o	c #BFA16A",
"p	c #19150D",
"q	c #876A31",
"r	c #C9A45B",
"s	c #C6A76A",
"t	c #7F6941",
"u	c #977737",
"v	c #D8B165",
"w	c #D8AC5B",
"x	c #BEA272",
"y	c #C09745",
"z	c #D8B163",
"A	c #927C52",
"B	c #735F3B",
"C	c #A68748",
"D	c #1A1A1A",
"E	c #393939",
"F	c #221C12",
"G	c #DDBC74",
"H	c #D2A64C",
"I	c #705627",
"                                                ",
"                                                ",
"                                                ",
"                                                ",
"                                                ",
"                                                ",
"                                                ",
"                                                ",
"                   ........                     ",
"                 ...+@#LJK...                   ",
"                ..#**=*---;J...                 ",
"               ..>,'*)>>>!-~J{..                ",
"               .>,]>^//(>>!_~J..                ",
"              .:*=><....[>>}~~|..               ",
"              .}'!<..   .1>!#~J..               ",
"             .2),#..    .3>}~#J..               ",
"             .4!>J..    .3}}_#5..               ",
"             .>!>J..    .3>__#6..               ",
"             .7~85..    .9}-#J...               ",
"             .......    0ab~#c..                ",
"              .....    .d)b#8...                ",
"                      .e}b##...                 ",
"                     ..>)fgh..i                 ",
"                     jklmnop..                  ",
"                    .qafrst..                   ",
"                    .u-vwx..                    ",
"                    .y-zwA..                    ",
"                    .#b~#B..                    ",
"                    .;-#8..                     ",
"                    .CCJ|..                     ",
"                    .DE...                      ",
"                     ....                       ",
"                                                ",
"                     F....                      ",
"                    .G!)8..                     ",
"                    .!**#..                     ",
"                    .))~8..                     ",
"                    .)_#8..                     ",
"                    .H~8I..                     ",
"                     ......                     ",
"                      ....                      ",
"                                                ",
"                                                ",
"                                                ",
"                                                ",
"                                                ",
"                                                ",
"                                                "]

  YES = true
  NO  = false

  #
  # initialize ()
  #
  #   aMessage : message string to show
  #   aPositon : specify  position of window
  #   aGrab    : specify button which grab focus by "YES" or "NO"
  #
  def initialize (aMessage, aTitle = '', aPosition = WIN_POS_MOUSE, aGrab = YES)
    super (aTitle, aMessage, XPM_QUESTION, aPosition)

    yes_button    = addButton("Ϥ", YES)
    no_button     = addButton("", NO)
    if aGrab == YES then
      yes_button.grab_default
    else
      no_button.grab_default
    end
  end
end



