# time_entry.rb

# $Id: time_entry.rb,v 1.1.1.1 2002/04/30 04:15:45 kenji Exp $

# 2002/4/30  Takuro Ashie <ashie@homa.ne.jp>
# Copyright (C) 2002 Takuro Ashie <ashie@homa.ne.jp>
# This is free software with ABSOLUTELY NO WARRANTY.

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.


###############################################################################
#
#   DateEdit Widget
#
###############################################################################
class DateEdit < Gtk::VBox
  include Gtk

  #
  # initialize ()
  #
  def initialize ()
    super (false, 0)

    @date_entry  = nil    # Gtk::Entry
    @date_button = nil    # Gtk::Button
    @cal_popup   = nil    # Gtk::Window
    @calendar    = nil    # Gtk::Calendar

    createChildren ()
  end

  attr_reader :date_entry
  attr_reader :date_button

  attr_reader :cal_popup
  attr_reader :calendar


  #
  # year () :
  #   Return year (Fixnum)
  #
  def year ()
    ymd = @date_entry.get_text.split ( /\// )
    if (ymd[0]) then
      return ymd[0].to_i
    else
      @calendar.year
    end
  end


  #
  # month () :
  #   Return month (Fixnum)
  #
  def month ()
    ymd = @date_entry.get_text.split ( /\// )
    if (ymd[1]) then
      return ymd[1].to_i
    else
      @calendar.month
    end
  end


  #
  # day () :
  #   Return day (Fixnum)
  #
  def day ()
    ymd = @date_entry.get_text.split ( /\// )
    if (ymd[2]) then
      return ymd[2].to_i
    else
      @calendar.day
    end
  end


  #
  # setDate () :
  #   set date to entry and calendar
  #
  # year  : ǯ (Fixnum)
  # month :  (Fixnum)
  # day   :  (Fixnum)
  #
  def setDate (year, month, day)
    @calendar.select_month (month, year)
    @calendar.select_day (day)
    @date_entry.set_text ("#{year}/#{month}/#{day}")
  end


 private
  #
  # createChildren () :
  #   create all children
  #
  def createChildren
    hbox = HBox::new (false, 0)
    pack_start (hbox,false, false, 0)
    hbox.show

    @date_entry = Entry::new
    @date_entry.set_usize (90, 0)
    hbox.pack_start (@date_entry, true, true, 0)
    @date_entry.show
    @date_entry.set_editable (false)

    @date_button = Button::new
    hbox.pack_start (@date_button, false, false, 0)
    @date_button.signal_connect ("clicked") {
      ymd = @date_entry.get_text.split(/\//)
      if ymd[0] && ymd[1] then
        @calendar.select_month (ymd[1].to_i, ymd[0].to_i)
      end
      if ymd[2] then
        @calendar.select_day (ymd[2].to_i)
      end
      @cal_popup.set_position (WIN_POS_MOUSE);
      @cal_popup.show
    }

    hbox1 = HBox::new (false, 3)
    @date_button.add (hbox1)
    hbox1.show

    @cal_label = Label::new ("")
    @cal_label.set_alignment (0.0, 0.5)
    hbox1.pack_start (@cal_label, true, true, 0)

    arrow = Arrow::new (ARROW_DOWN, SHADOW_OUT)
    hbox1.pack_start (arrow, false, false, 0)
    arrow.show

    @date_button.show

    time = Time::now
    @calendar = Calendar::new
    @calendar.select_month (time.month, time.year)
    @calendar.select_day (time.day)
    calendar.show
    @calendar.signal_connect ("day_selected") {
      @date_entry.set_text ("#{@calendar.year}/#{@calendar.month}/#{@calendar.day}")
    }
    calendar.signal_connect ("day_selected_double_click") {
      @cal_popup.hide
    }

    @cal_popup = Window::new (WINDOW_POPUP)
    @cal_popup.add (@calendar)
      
  end
end



###############################################################################
#
#   TimeEntry Widget
#
###############################################################################
class TimeEntry < Gtk::HBox
  include Gtk

  #
  # initialize ()
  #
  def initialize ()
    super (false, 0)

    @date_edit  = nil  # Gtk::DateEdit
    @hour_spin  = nil  # Gtk::SpinButton
    @min_spin   = nil  # Gtk::SpinButton
    @sec_spin   = nil  # Gtk::SpinButton

    createChildren
    setTime (Time::now)
  end


  #
  # getTime () :
  #    Create new Time object using each entry's value.
  #
  # Return : New Time Object
  #
  def getTime ()
    return Time::local (@date_edit.year, @date_edit.month, @date_edit.day,
                        @hour_spin.get_text, @min_spin.get_text,@sec_spin.get_text)
  end


  #
  # setTime () :
  #
  # time : Time to set (Time Object)
  #
  def setTime (aTime)
    @date_edit.setDate (aTime.year, aTime.month, aTime.day)
    @hour_spin.set_value (aTime.hour)
    @min_spin.set_value (aTime.min)
    @sec_spin.set_value (aTime.sec)
  end

 private

  #
  # createChildren () :
  #   create all children
  #
  def createChildren ()
    @date_edit = DateEdit::new
    pack_start (@date_edit, false, false, 0)
    @date_edit.show

    adj = Adjustment::new (0.0, 0.0, 23.0, 1.0, 5.0, 0.0)
    @hour_spin = SpinButton::new (adj, 0, 0)
    pack_start (@hour_spin, false, false, 0)
    @hour_spin.show
    label = Label::new ('')
    pack_start (label, false, false, 3)
    label.show

    adj = Adjustment::new (0.0, 0.0, 59.0, 1.0, 5.0, 0.0)
    @min_spin = SpinButton::new (adj, 0, 0)
    pack_start (@min_spin, false, false, 0)
    @min_spin.show
    label = Label::new ('ʬ')
    pack_start (label, false, false, 3)
    label.show

    adj = Adjustment::new (0.0, 0.0, 59.0, 1.0, 5.0, 0.0)
    @sec_spin = SpinButton::new (adj, 0, 0)
    pack_start (@sec_spin, false, false, 0)
    @sec_spin.show
    label = Label::new ('')
    pack_start (label, false, false, 3)
    label.show
  end
end
